#!/usr/bin/python3.5
#Version 34
import requests
from requests.packages.urllib3.exceptions import InsecureRequestWarning
import xml.etree.ElementTree as ET
import json

import sys
import argparse
import os

# A subdirectory for verbose output and files
path='debug'

# Used for logon and HMC comms authority
HMCname = ''
token = ''
tokenFilename = "HMC.token"

def save_to_file(filename, string):
   if debug:
     filename = path + '/' + filename
     file = open(filename,"w")
     file.write(string)
     file.close()
     print("------- Saved data to file " + filename)


def HMClogon(user, passwd, reuse):
    logonheaders = {'Content-Type': 'application/vnd.ibm.powervm.web+xml; type=LogonRequest'}
    logonUrl =  'https://'+HMCname+':12443/rest/api/web/Logon'
    logonPayload = '<LogonRequest schemaVersion="V1_0" xmlns="http://www.ibm.com/xmlns/systems/power/firmware/web/mc/2012_10/" xmlns:mc="http://www.ibm.com/xmlns/systems/power/firmware/web/mc/2012_10/"><UserID>' + user + '</UserID><Password>' + passwd + '</Password></LogonRequest>'
    ret = requests.put(logonUrl,data=logonPayload,headers=logonheaders,verify=False)
    if ret.status_code != 200:
            print("Error: Logon failed error code=%d url=%s" %(ret.status_code,logonUrl))
            if debug:
                    print("Returned:" + ret.text)
            # Do not return if we failed to logon
            sys.exit(ret.status_code)
    #Extract from the XML the token
    xmlResponse = ET.fromstring(ret.text)
    token = xmlResponse[1].text
    if debug:
            print("Log on response=%d and got Token=\n----\n%s\n----" %(ret.status_code, token))
    if reuse:
        print("Save session token in local directory file:%s" %(tokenFilename))
        file_object = open(tokenFilename,"w")
        file_object.write(token)
        file_object.close()
    return token

def HMClogoff():
    if reuse:
        print("Leave %s file for next time" %(tokenFilename))
        sys.exit(0)
    logoffheaders = {'X-API-Session' : token }
    logoffUrl =  'https://'+HMCname+':12443/rest/api/web/Logon'
    ret = requests.delete(logoffUrl,headers=logoffheaders,verify=False)
    rcode = ret.status_code
    # delete can respond with these three good values
    if rcode == 200 or rcode == 202 or rcode == 204:
        print("Successfully disconnected from the HMC (code=%d)" %(rcode))
        if os.path.isfile(tokenFilename):
            os.remove(tokenFilename)
        sys.exit(0)
    else:
        print("Error: Logoff failed error code=%d url=%s data=%s" %(rcode, logoffUrl, ret.text))
        sys.exit(rcode)

def get_preferences():
    print("------- Get Preferences")
    prefHeaders = {'X-API-Session' : token}
    prefUrl = 'https://'+HMCname+':12443/rest/api/pcm/preferences/SSP'
    ret = requests.get(prefUrl, headers=prefHeaders,verify=False)
    if debug:
        save_to_file("SSP-prefs.xml", ret.text)
    if ret.status_code != 200:
        print("Error: GET preferences error code=%d header=%s url=%s" %(ret.status_code, preHeaders, prefUrl))
        if ret.status_code == 401:
                print("Unauthorisied either wrong User/Password wrong or HMC.token out of date")
        if debug:
                print("Returned: data=%s" %( ret.text))
        return ""
    return ret.text

def set_preferences(postBody):
    print("------- Update Preferences")
    # Note that currently the prefereences have AggregationEnabled=true as a default - but we may not need that
    # It was remommended to force it anyway, so we set two flags to true
    # Set the flags to true for each SSP
    # Note: kb= and kxe= are in different orders!! But we can strip them out
    postBody = postBody.replace(' kb="UOD"',   '')
    postBody = postBody.replace(' kb="ROO"',   '')
    postBody = postBody.replace(' kb="ROR"',   '')
    postBody = postBody.replace(' kxe="false"','')
    postBody = postBody.replace(' kxe="true"', '')
    # Set AggregationEnabled=true & MonitorEnabled=true
    # Not clear if the Aggregate flag is really needed for ProcessedMetrics
    postBody = postBody.replace('<AggregationEnabled>false</AggregationEnabled>',\
                                '<AggregationEnabled>true</AggregationEnabled>')
    postBody = postBody.replace('<MonitorEnabled>false</MonitorEnabled>',\
                                '<MonitorEnabled>true</MonitorEnabled>')
    # if the POST includes the first and last few lines of XML it fails completely
    # rip off the pre-amble
    i = postBody.find("<ManagementConsolePCMSSPPreference")
    postBody = postBody[i : ]
    # rip off the trailer
    i = postBody.find("</content>")
    postBody = postBody[ :i ]
    if debug:
        save_to_file ("SSP-prefs-updated.xml", postBody)
    postHeaders = {'Content-Type': 'application/xml', 'X-API-Session' : token }
    postUrl = 'https://'+HMCname+':12443/rest/api/pcm/preferences/SSP'
    r = requests.post(postUrl, headers=postHeaders, data=postBody, verify=False)
    if r.status_code != 200:
        print("Error: post preferences error code=%d header=%s url=%s" %( r.status_code,postHeaders,postUrl))
        if debug:
                print("URL::" + postUrl)
                print("Body:" + postBody)
                print("Returned:" + r.text)
        HMClogoff()

def get_filenames(SSPid, SSP):
    fileHeaders = {'X-API-Session' : token, 'Accept': 'application/atom+xml' }
    fileUrl =  'https://'+HMCname+':12443/rest/api/pcm/SharedStoragePool/'+SSPid+'/ProcessedMetrics' 
    r = requests.get(fileUrl,headers=fileHeaders,verify=False)
    if r.status_code != 200:
            print("Error %d: Get SSP %s file of filenames \n\tHeader=%s \n\tURL=%s" \
                  %( r.status_code,SSP,fileHeaders,fileUrl))
            if r.status_code == 204:
                print("Hint: 204=No Content - Have you updated the Preferences to switch on monitoring? See --prefs option")
                print("Hint: 204=No Content - Wait 6 minutes after you switch on stats to get some data")
            if debug:
                print("Returned:" + r.text)
            print("Continue to next SSP")
            return ""
    if debug:
            file_name = "SSP-filenames-" + SSP + ".xml"
            save_to_file(file_name, r.text)
    return r.text

def get_stats(JSON_url, JSON_file, SSP):
    jsonHeaders = {'X-API-Session' : token }
    r = requests.get(JSON_url,headers=jsonHeaders,verify=False)
    if r.status_code != 200:
        print("Error: get SSP JSON file error code=%d filename=%s" %( r.status_code, JSON_file))
        if debug:
                print("Returned:" + r.text)
        return ""
    if debug:
        file_name = "SSP-ProcessedMetrics-" + SSP + "-" + JSON_file
        save_to_file(file_name, r.text)
    return r.text

#### MAIN
parser = argparse.ArgumentParser()
parser.add_argument("--hmc", help="Specify the HMC IP Address or Hostname")
parser.add_argument("--username", help="Username for the HMC")
parser.add_argument("--password", help="Password for the HMC")
parser.add_argument("--ssp", help="SSP name or ALL(default)")
parser.add_argument("--debug", help="Output details as we go + files in directory debug",action="store_true")
parser.add_argument("--prefs", help="Update the SSP preferences (only needed once)",action="store_true")
parser.add_argument("--reuse", help="Save and reuse HMC token",action="store_true")
# extract the parameters
args = parser.parse_args()
HMCname = args.hmc
userName = args.username
password = args.password
sspNeeded = args.ssp
debug = args.debug
prefs = args.prefs
reuse = args.reuse
# check the parameters
if(userName == None):
        sys.exit('HMC --username  is mandatory')
if(password == None):
        sys.exit('HMC --password  is mandatory')
if(HMCname == None):
        sys.exit('HMC --hmc <IP or host name>  is mandatory')
if(sspNeeded == None):
        sspNeeded = "ALL"
if(prefs == None):
        prefs = 0

if debug:
    print("Switching off ugly Security Warnings 'Unverified HTTPS request is being made'")
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

print("------- SSP names: %s   Set Preferences:%d Debug:%d Reuse:%d (0=No)" %(sspNeeded, prefs, debug, reuse))
# if --reuse load the token file and try it. It could be invalid or too old
response = ""
if reuse:
    if os.path.isfile(tokenFilename) and os.access(tokenFilename, os.R_OK):
        print("HMC.token exists and is readable")
        file_object = open(tokenFilename,"r")
        token = file_object.read()
        file_object.close()
        # Test the token by extracting the preferences
        response = get_preferences()
        if response == "":
            print("HMC.token file did not work")
            os.remove(tokenFilename)
        else:
            print("HMC.token file worked")
    else:
        print("No readable HMC.token file found")
# if --reuse failed or not --reuse try a full logon
if response == "":
    print("HMC Logon")
    token = HMClogon(userName, password, reuse)
    response = get_preferences()
# if we can't communicate with the HMC stop here
if response == "":
    print("HMC Login or Get SSP Preferences failed ... stopping")
    HMClogoff()
# if --prefs then set the data needed flags 
if (prefs):
    set_preferences(response)
    print("------- Successfully updated Preferences")
else:
    print("------- Not Updating Preferences")

# Convert the GET preferenses response TEXT to XML 
#    So we can extract the Management Server UUID
xmlResponse = ET.fromstring(response) # 

print("Looking for Shared Storage Pool: %s" %(sspNeeded))
SSPName = ""
for feed in xmlResponse:
    if 'entry' in feed.tag:
        for entry in feed:
            if 'content' in entry.tag:
                for content in entry:
                  if 'ManagementConsolePCMSSPPreference' in content.tag:
                    for pcm in content:
                        if 'Metadata' in pcm.tag:  # skip over the empty Metadat entry
                            continue
                        if 'ManagementConsoleSSPPreference' in pcm.tag:
                            # we have a SSP record so pick out the details we need
                            # then request the filenames using the SSPUuId
                            for ssp in pcm:
                                if 'ClusterName' in ssp.tag:
                                        ClusterName = ssp.text
                                if 'ClusterUdId' in ssp.tag:
                                        ClusterUdId = ssp.text
                                if 'SSPName' in ssp.tag:
                                        SSPName = ssp.text
                                if 'SSPUuId' in ssp.tag:
                                        SSPUuId = ssp.text
                                if 'AggregationEnabled' in ssp.tag:
                                        AggregationEnabled = ssp.text
                                if 'MonitorEnabled' in ssp.tag:
                                        MonitorEnabled = ssp.text
                            if SSPName != "":  # we found a SSP
                                print("SSP=%s ClusterName=%s\n\tSSPUuId=%s\n\tAggregationEnabled=%s MonitorEnabled=%s" % \
                                        (SSPName,ClusterName,SSPUuId,AggregationEnabled,MonitorEnabled))
                            if sspNeeded == SSPName or sspNeeded == "ALL":
                                refResponse = get_filenames(SSPUuId, SSPName)
                                if refResponse == "":
                                    continue  # get_filename() returned no data
                                # Parse the file of filenames
                                refResponse = ET.fromstring(refResponse)
                                for reffeed in refResponse:
                                    if 'entry' in reffeed.tag:
                                        for entry in reffeed:
                                            if 'title' in entry.tag:
                                                filename = entry.text
                                            if 'link' in entry.tag:
                                                JSON_file_url = entry.get('href')
                                        if debug:
                                                print("JSON filename |%s|" % (filename))
                                                print("JSON URL |%s|" % (JSON_file_url))
                                        statResponse = get_stats(JSON_file_url, filename, SSPName)
                                        # convert to JSON
                                        if statResponse == "":
                                                    print("Getting JOSN stats faile failed continuing ...")
                                                    continue

                                        data = json.loads(statResponse)

                                        sspName        = data["sspUtil"]["utilInfo"]["name"]
                                        clusterName    = data["sspUtil"]["utilInfo"]["clusterName"]
                                        frequency      = data["sspUtil"]["utilInfo"]["frequency"]
                                        startTimeStamp = data["sspUtil"]["utilInfo"]["startTimeStamp"]
                                        endTimeStamp   = data["sspUtil"]["utilInfo"]["endTimeStamp"]
                                        metricType     = data["sspUtil"]["utilInfo"]["metricType"]
                                    
                                        print("\tSSP:%s cluster=%s frequency=%s type=%s" % \
                                            (sspName,clusterName,frequency,metricType ))
                                        print("\tSSP:%s Start=%s" % (sspName,startTimeStamp))
                                        print("\tSSP:%s End  =%s" % (sspName,endTimeStamp))

                                        file_name = "ssp_total_io.csv"
                                        file_object = open(file_name,"a")

                                        vios_name = "ssp_vios_io.csv"
                                        vios_object = open(vios_name,"a")
                                  
                                        firsttime   = 1 # used to print only one header line per run
                                        samplecount = 0
                                        vioscount   = 0
                                        for sample in data["sspUtil"]["utilSamples"]:
                                            # Each sample is a stats snapshot for this SSP
                                            samplecount += 1
                                            timeStamp   = sample['sampleInfo']['timeStamp']
                                            free        = sample['poolUtil']['free'][0]
                                            size        = sample['poolUtil']['size'][0]
                                            numOfReads  = sample['poolUtil']['numOfReads'][0]
                                            numOfWrites = sample['poolUtil']['numOfWrites'][0]
                                            readBytes  = sample['poolUtil']['readBytes'][0]
                                            writeBytes = sample['poolUtil']['writeBytes'][0]
                                            readServiceTime = sample['poolUtil']['readServiceTime'][0]
                                            writeServiceTime = sample['poolUtil']['writeServiceTime'][0]

                                            if debug:
                                                print("%s size=%d free=%d OPs:read=%6d write=%6d Bytes:read=%9d write=%9d ServiceTime:read=%9d write=%9d" % (timeStamp,size,free,numOfReads,numOfWrites,readBytes,writeBytes,readServiceTime,writeServiceTime))
                                            # Google chart Javascript Months start from 0 = January hence the -1
                                            buffer = "%s,%s-%02d-%s-%s-%s-%s,%d,%d,%d,%d,%d,%d,%.2f,%.2f\n" % (sspName,timeStamp[0:4],int(timeStamp[5:7]) -1, timeStamp[8:10], timeStamp[11:13], timeStamp[14:16],timeStamp[17:19],size,free,numOfReads,numOfWrites,readBytes,writeBytes,float(readServiceTime)/1000000.0,float(writeServiceTime)/1000000.0)
                                            file_object.write(buffer)

                                            if firsttime:
                                                print("\tSSP:%s Collecting VIOS data"%(sspName))
                                            index=0
                                            nname=[]
                                            nrbytes=[]
                                            nwbytes=[]
                                            for node in sample['poolUtil']['nodeUtil']:
                                                # Each node is a SSP VIOS with its own detailed stats 
                                                vioscount += 1
                                                #nname = node['name']
                                                nname.append(node['name'])
                                                #nname[index] = nname[index].replace('.aixncc.uk.ibm.com', '')
                                                npool = node['poolState'] # "UP or "DOWN"
                                                nrbytes.append(node['readBytes'][0])
                                                nwbytes.append(node['writeBytes'][0])
                                                #if npool != "UP":
                                                    #print("VIOS %s state is %s server is a %s" %(name,npool,nmtms))
                                                index += 1
                                            if firsttime == 1:
                                                firsttime = 0
                                                #print("%s, Header" %(clusterName), end='')
                                                buffer = "%s,Header" %(clusterName)
                                                for name in nname:
                                                    name = name.replace('.aixncc.uk.ibm.com', '')
                                                    #print(", %s-Read-KBs" % (name), end='')
                                                    buffer = buffer + ",%s-Read-KBs" % (name)
                                                for name in nname:
                                                    name = name.replace('.aixncc.uk.ibm.com', '')
                                                    #print(", %s-Write-KBs" % (name), end='')
                                                    buffer = buffer + ",%s-Write-KBs" % (name)
                                                #print("")
                                                buffer = buffer + "\n"
                                                if debug:
                                                    print(buffer)
                                                vios_object.write(buffer)
                                            #print("%s, %s" %(timeStamp,clusterName), end='')
                                            # Google chart Javascript Months start from 0 = January hence the -1
                                            buffer = "%s,%s-%02d-%s-%s-%s-%s" %(clusterName,timeStamp[0:4],int(timeStamp[5:7]) -1, timeStamp[8:10], timeStamp[11:13], timeStamp[14:16],timeStamp[17:19])
                                            for bytes in nrbytes:
                                                #print(", %d" % (bytes/1024), end='')
                                                buffer = buffer + ",%d" % (bytes/1024)
                                            for bytes in nwbytes:
                                                #print(", -%d" % (bytes/1024), end='')
                                                buffer = buffer + ",-%d" % (bytes/1024)
                                            #print("")
                                            buffer = buffer + "\n"
                                            if debug:
                                                print(buffer)
                                            vios_object.write(buffer)
                                            # NOTE: we could add tier stats at this place BUT we would need to 
                                            # add the stats for every VIOS together to get useful stats
                                            # This is possible if we use a Python array
                                            # Otherwise 24 VIOS nodes and 9 teirs and read and write would give us 432 lines on a graph
                                            # which is completely hopeless.
                                        file_object.close()
                                        vios_object.close()
                                        print("\tSSP:%s Saved %d samples and %d VIOS samples" %(SSPName,samplecount,vioscount))
                                        print("\tSSP:%s SSP size=%.2f GB Used=%.2f GB Free=%.2f GB =%.2f%%" 
                                                        %(SSPName, float(size)/1024.0,  float(size - free)/1024.0, float(free)/1024.0, float(free)/float(size)*100.0))
                        else:
                            print("        Ignoring the SSP" + pcm.tag)

HMClogoff()
