#!/usr/bin/python3.5
#Version 34
import os
import csv

def clean_data(source, temp, skip):
     os.system('sort ' + source + ' | uniq | grep -v "' + skip + '" > ' + temp)
     os.system('cp ' + temp + ' ' + source)

# Head of the webapge
def output_start(file, sspname):
    file.write('<html>' + '\n')
    file.write('<head>' + '\n')
    file.write('\t<title>SSP ' + sspname +  'Stats</title>' + '\n')
    file.write('\t<script type="text/javascript" src="https://www.google.com/jsapi"></script>' + '\n')
    file.write('\t<script type="text/javascript">' + '\n')
    file.write('\tgoogle.load("visualization", "1.1", {packages:["corechart"]});' + '\n')
    file.write('\tgoogle.setOnLoadCallback(setupCharts);' + '\n')
    file.write('\tfunction setupCharts() {' + '\n')
    file.write('\tvar chart = null;' + '\n')

#Before data
def output_top(file, graphnum, units1, units2):
    file.write('\tvar data_' + str(graphnum) +  ' = google.visualization.arrayToDataTable([' + '\n')
    # opposite use of ' and " as e need to out '
    file.write("[{type: 'datetime', label: 'Datetime'},'" + units1 + "','" + units2 + "']\n")

#Before data but units are supplied as there is a variable number of them
def output_top_no_units(file, graphnum):
    file.write('var data_' + str(graphnum) + ' = google.visualization.arrayToDataTable([\n')

#After  data
def output_bot(file, graphnum, sspname, description):
    file.write('\t]);\n')
    file.write('\tvar options_'+ str(graphnum) + ' = {\n')
    file.write('\t\tchartArea: {left: "5%", width: "85%", top: "10%", height: "80%"},\n')
    file.write('\t\ttitle: "Shared Storage Pool:' + sspname + ' - ' + description + ' ",\n')
    file.write('\t\tfocusTarget: "category",\n')
    file.write('\t\thAxis: { gridlines: { color: "lightgrey", count: 30 } },\n')
    file.write('\t\tvAxis: { gridlines: { color: "lightgrey", count: 11 } },\n')
    file.write('\t\texplorer: { actions: ["dragToZoom", "rightClickToReset"],\n')
    file.write('\t\taxis: "horizontal", keepInBounds: true, maxZoomIn: 20.0 },\n')
    file.write('\t\tisStacked:  0\n')
    file.write('\t};\n')
    file.write('\tdocument.getElementById("draw_'+ str(graphnum) + '").addEventListener("click", function() {\n')
    file.write('\tif (chart && chart.clearChart) chart.clearChart();\n')
    file.write('\tchart = new google.visualization.AreaChart(document.getElementById("chart_master"));\n')
    file.write('\tchart.draw( data_'+ str(graphnum) + ', options_'+ str(graphnum) + ');\n')
    file.write('\t});\n')

#Finish off the web page
def output_end(file, sspname):
    file.write('\t}\n')
    file.write('\t</script>\n')
    file.write('\t</head>\n')
    file.write('\t<body bgcolor="#EEEEFF">\n')
    file.write('\t<b>Shared Storage Pool: ' + sspname + ': </b>\n')
    file.write('\t<button id="draw_1" style="color:red;"><b>SSP Capacity MB</b></button>\n')
    file.write('\t<button id="draw_2" style="color:blue;"><b>Disk Throughput KB/s</b></button>\n')
    file.write('\t<button id="draw_3" style="color:green;"><b>Disk Operations/s</b></button>\n')
    file.write('\t<button id="draw_4" style="color:brown;"><b>Disk Service Time</b></button>\n')
    file.write('\t<button id="draw_5" style="color:purple;"><b>VIOS KB/s</b></button>\n')
    file.write('\t<div id="chart_master" style="width:100%; height:85%;">\n')
    file.write('\t<h2 style="color:blue">Click on a Graph button above, to display that graph</h2>\n')
    file.write('\t</div>\n')
    file.write('\n')
    file.write('Author: Nigel Griffiths @mr_nmon generated by SPPIO.py\n')
    file.write('Now with Zoom = Left-click and drag. To Reset = Right-click.\n')
    file.write('</body>\n')
    file.write('</html>\n')

# ----------------

clean_data("ssp_total_io.csv", "ssp_total_io_clean.csv", "0,0,0,0,0.00,0.00")
clean_data("ssp_vios_io.csv",  "ssp_vios_io_clean.csv",  ",0,0,")

#Generate a list of SSP names and add rows to a list
SSPnames = set()
SSPall = []
VIOSall = []
VIOSheader = []

with open('ssp_total_io.csv') as f:
    f_csv = csv.reader(f)
    headers = next(f_csv)
    for row in f_csv:
        SSPall.append(row)
        SSPnames.add(row[0])

with open('ssp_vios_io.csv') as f:
    f_csv = csv.reader(f)
    headers = next(f_csv)
    for row in f_csv:
        VIOSall.append(row)

# Take each SSP one at a time
i = 1
for SSP in SSPnames:
    print("processing SSP %d ->%s<" %(i,SSP))
    i = i + 1
    # Extract this single SSP's data to a different list
    sspone = []
    for row in SSPall:
        if SSP == row[0]:
            sspone.append(row)

    viosone = []
    viosheader = []
    for vios in VIOSall:
        if SSP == vios[0]:
            if vios[1] == 'Header':
                viosheader = vios
                print('Saving header:' + str(viosheader))
            else:
                if vios[0] == SSP:
                    viosone.append(vios)

    # Open the web page file
    web = open('ssp_' + SSP + '.html', "w")
    output_start(web, SSP)

    output_top(web, 1, 'Size-MB', 'Free-MB') 
    for row in sspone:
            web.write(',[\'Date(' + row[1].replace('-',',')  + ')\',' + row[2] + ',' + row[3]  + ']\n')
    output_bot(web,  1, SSP, "SSP Capacity in MB")

    output_top(web, 2, 'Read-KB/s', 'Write-KB/s') 
    for row in sspone:
            web.write(',[\'Date(' + row[1].replace('-',',')  + ')\',' + row[4] + ',' + row[5]  + ']\n')
    output_bot(web,  2, SSP, 'Disk Throughput KB per second')

    output_top(web, 3, 'Read-Ops', 'Write-Ops')
    for row in sspone:
            web.write(',[\'Date(' + row[1].replace('-',',')  + ')\',' + row[6] + ',' + row[7]  + ']\n')
    output_bot(web,  3, SSP, 'Disk Operations per second')

    output_top(web, 4, 'Read-Service-time-msec', 'Write-Service-Time-msec')
    for row in sspone:
            web.write(',[\'Date(' + row[1].replace('-',',')  + ')\',' + row[8] + ',' + row[9]  + ']\n')
    output_bot(web,  4, SSP, 'Disk Service Time in milli-seconds')

    output_top_no_units(web, 5)
    # Remove the first two coloumns: SSP name and Header
    viosheader.remove(viosheader[0])
    viosheader.remove(viosheader[0])
    web.write('[{type: \'datetime\', label: \'Datetime\'}')
    for name in viosheader:
        web.write(", '" + name + "'")
    web.write(']\n')
    for vios in viosone:
        # Remove SSP name column
        vios.remove(vios[0]) # Remove the SSP name column
        web.write(',[\'Date(' + vios[0].replace('-',',')  + ')\',')
        # Remove date time
        vios.remove(vios[0]) # remove the Data column
        line = str(vios)
        line = line.replace('[','')
        line = line.replace('\'','')
        line = line.replace('"','')
        line = line.replace(']','')
        web.write(line  + ']\n')
    output_bot(web,  5, SSP, 'VIOS KB/s with read above zero and write below zero')

    # Finish web page with buttons etc.
    output_end(web,  SSP)
    web.close()

